<?php

namespace InSegment\WorkersFramework\Models;

use Shavshukov\EventAppBase\FileReaders\FileReaderInterface;
use Shavshukov\EventAppBase\Storage\FileStorage;
use Shavshukov\EventAppBase\Model\YamlModel;

class State extends YamlModel
{
    
    /**
     * File name of the open state file
     *
     * @var string
     */
    protected $fileName;
    
    /**
     * Constructor
     * 
     * @param \Shavshukov\EventAppBase\FileReaders\FileReaderInterface $fileReader
     * @param string $fileName
     */
    public function __construct(FileReaderInterface $fileReader, string $fileName)
    {
        $this->fileName = $fileName;
        $fileStorage = new FileStorage($fileReader, $fileName);
        $this->backend = $fileStorage;
    }
    
    /**
     * Get file name
     * 
     * @return string
     */
    public function getFileName(): string
    {
        return $this->fileName;
    }
    
    /**
     * Get whether in the state model, the file of the worker is locked
     * 
     * @param string $workerKey
     * @return bool
     */
    public function isWorkerLocked(string $workerKey): bool
    {
        return !empty($this->data['locked']) || !empty($this->data['workers'][$workerKey]['locked']);
    }

}
