<?php

namespace InSegment\WorkersFramework\Services;

use Shavshukov\EventAppBase\ApplicationInterface;
use React\ChildProcess\Process;

use InSegment\WorkersFramework\Models\State;
use InSegment\WorkersFramework\Queues\Worker;

class WorkerProcess
{
    /**
     * Application instance
     * 
     * @var \Shavshukov\EventAppBase\ApplicationInterface $app 
     */
    protected $app;
    
    /**
     * State model instance
     *
     * @var \InSegment\WorkersFramework\Models\State $state 
     */
    protected $state;
    
    /**
     * Worker instance
     *
     * @var \InSegment\WorkersFramework\Queues\Worker 
     */
    protected $worker;
    
    /**
     * Process instance
     * 
     * @var \React\ChildProcess\Process
     */
    protected $process;
    
    /**
     * Constructor
     * 
     * @param \Shavshukov\EventAppBase\ApplicationInterface $app
     * @param \InSegment\WorkersFramework\Models\State $state
     * @param \InSegment\WorkersFramework\Queues\Worker $worker
     */
    public function __construct(ApplicationInterface $app, State $state, Worker $worker)
    {
        $this->app = $app;
        $this->state = $state;
        $this->worker = $worker;
        $this->process = $this->createProcess();
    }
    
    /**
     * Start the defied child process
     * 
     * @return null
     */
    public function start()
    {
        if ($this->worker->isActive()) {
            echo "Already active process for worker: {$this->worker->getName()}";
            return;
        }
        
        $isLocked = $this->state->isWorkerLocked($this->worker->getName());
        if ($isLocked) {
            $modify = [
                'active' => false,
                'workerState' => 'offline',
                'startTime' => null
            ];
            echo "Consumer {$this->worker->getName()} is Locked. \n";
        } else {
            echo "> {$this->process->getCommand()}\n";
            $this->process->start($this->app->getLoop());
            $modify = [
                'active' => true,
                'workerState' => 'consuming',
                'startTime' => time()
            ];
        }
        
        $this->worker->modifyStatus($modify);
    }
    
    /**
     * Get command to run worker
     * 
     * @param string $arg
     * @return string
     */
    protected function getWorkerCommand($arg): string
    {
        return sprintf($this->app->config('worker_run_command'), $arg);
    }
    
    /**
     * Create child process definition for worker
     * 
     * @return \InSegment\WorkersFramework\Services\Process
     */
    protected function createProcess()
    {
        $workerName = $this->worker->getName();
        $commandLine = $this->getWorkerCommand(escapeshellarg($workerName));
        $process = new Process('exec ' . $commandLine, null, null, array());
        $process->on('exit', function ($exitCode, $termSignal) use ($process, $workerName) {
            echo "Process work {$workerName} exited with code {$exitCode} \n";
            $modify = ['active' => false];
            
            if ($exitCode !== 0) {
                $modify['workerState'] = 'error';
            }
            
            $modify['errorCode'] = $exitCode;
            $this->worker->modifyStatus($modify);
            $this->start();
        });
        
        return $process;
    }
}
